#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::addattachment;

use strict;
use warnings;

require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast);
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;
use Encode;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(addattachment);

our $VERSION = '0.01';

#######################################################################
our @ARGVBAACKUP = @ARGV;
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => [ ], 'cmdname'  => "add_attachment");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
			 'user'		=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;
our	@FileList = undef;

our	$RIdDef = undef;
our $DbidDef = undef ;
our	$TypeDef = "";
our	$FieldName="Attachments";
our	$ActionDef = "Modify";
our $Comment = undef;

our %FileCommentAssociation = ();

#########################################################################

#########handle getopt warnings begin########################

sub InitParser
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "add_attachment");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	@FileList = undef;
	
	%FileCommentAssociation=();
	
	$RIdDef = undef;
	$DbidDef = undef ;
	
	$TypeDef = "";
	$ActionDef = "Modify";
	$FieldName="Attachments";
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub CleanUpParser()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "add_attachment");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	@FileList = undef;	

	$RIdDef = undef;
	$DbidDef = undef ;	
	
	$TypeDef = "";
	$ActionDef = undef;
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;
	%FileCommentAssociation=();
}

#########handle getopt warnings end#####################################

sub handle_help_usage
{
	my $opt = $_[0];
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}


sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_action_name
{
	my ($opt , $val) = @_;
	my $msg = undef;

	if($RepetitionCount{'handle_action_name'}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_action_name'} = 1;
	}


	if( ($opt eq "action") && ($val)	){
		$ActionDef=$val;
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;

}

sub handle_recordtype_list
{
	my ($opt , $type) = @_;
	my $msg = undef;
	if($RepetitionCount{'$opt'} ) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'$opt'} = 1;
	}

	if($type){
		$TypeDef = $type;
	}else {
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt", $CmdInfo{'cmdname'} );
	}
	return 1;
}

sub handle_recordid_list
{
	my ($opt , $val) = @_;

	if( ($opt eq "record") && ($val) ){
		$RIdDef = $val;
	}elsif(($opt eq "dbid") && ($val)){
		$DbidDef = $val;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;

}

sub handle_comment_name
{
	my ($opt , $val) = @_;
	my $msg = undef;

	if($RepetitionCount{'handle_comment_name'}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_comment_name'} = 1;
	}


	if( ($val)	){
		$Comment=$val;
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;

}

sub handle_field_name
{
	my ($opt , $val) = @_;
	my $msg = undef;

	if($RepetitionCount{'handle_field_name'}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_field_name'} = 1;
	}


	if( ($val)	){
		$FieldName=$val;
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForInterActiveOption 
{
	my $ret = undef;
	my $askRules = {'ask!'	=>	\&handle_ask_mode	};
	Getopt::Long::Configure('posix_default','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$askRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|login|u|l:s'					=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}

sub parseForCoreOptions()
{
	my $ret = undef;
	my $CoreOptRules = {
		'type:s'   =>		\&handle_recordtype_list, 
		'action:s' =>   	\&handle_action_name , 
		'field:s'  =>   	\&handle_field_name ,
		'comment|co:s' =>   \&handle_comment_name ,
		'record:s' => \&handle_recordid_list,
		'dbid:s' => \&handle_recordid_list
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForUnknownOptions
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	my $msg = undef;
	my $fname = undef;
	my $checkfile = undef;

	if($lmt == -1) {
		CheckPromptModeValidity('CQCLI_Missing_File_Name');
		if(AskForYN('CQCLI_Prompt_AskFileName')) {
			$fname = AskForFileName("add_attachment");
			
			if(!(-e $fname) ) {
			ThrowSyntaxErrorException("CQCLI_Missing_File",$fname,$CmdInfo{'cmdname'});
			}

			$FileCommentAssociation{$fname} = $Comment;
			return 1;
		}
		else {
		ThrowSyntaxErrorException("CQCLI_Missing_Value","File Name",$CmdInfo{'cmdname'});
		}
	}
	
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		$FileList[$cnt] = $tmpARGV[$cnt];
		$checkfile = $tmpARGV[$cnt];

		#######################################################

		if($ENV{CQCLI_LOCALE} eq 'zh') {
			$checkfile = encode('cp936',$checkfile);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'jp') {
			$checkfile  = encode('cp932',$checkfile);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'en') {
			$checkfile  = encode('cp1252',$checkfile);
		}

		if(!(-e $checkfile)){
			ThrowException('CQCLI_Failure_FileOpen',$checkfile);
		}

		########################################################

		
		if(!(-e $checkfile) ) {
			ThrowSyntaxErrorException("CQCLI_Missing_File",$FileList[$cnt],$CmdInfo{'cmdname'});
		}
		else {
		$FileCommentAssociation{$FileList[$cnt]} = $Comment;
		}
		
	}
	
	
	
	return 1;
}

sub ValidateSessionInfo()
{
	if($isLoginOpt){
		ValidateLoginCredentials();
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
	      $JavaReqObj->put("type",$TypeDef);
		return;
	}elsif($CliSession){
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
	      $JavaReqObj->put("type",$TypeDef);
		return;
	}else{
		ValidateLoginCredentials();
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
	      $JavaReqObj->put("type",$TypeDef);
		return;
	}
}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}

sub ValidateActionNames
{

	if(!($ActionDef)){
		$ActionDef = GetActionName($RIdDef,$TypeDef,1);
	}else {
		my $action = $ActionDef;
		my $recordinfo = GetRecordInfoObj();
		my $actionlist = $recordinfo->getLegalActions($RIdDef,$TypeDef,1);
		$actionlist = lc(cqclitrim($actionlist));
		$actionlist = $actionlist.",";
		if(!( $actionlist =~/\s*\,?$action\s*\,/i )){
			ThrowValidationException('CQCLI_Failrure_Action',$action);
		}
	}

	$JavaReqObj->put("action",$ActionDef);	
}

sub ValidateFieldNames()
{
	my $recordtypeobj = GetRecordTypeInfoObj($TypeDef);
	my $expectedfield = $recordtypeobj->isValidFieldName($FieldName);
	
	
	if($expectedfield) {
		if($expectedfield =~ /\,/ ){
			ThrowValidationException("CQCLI_Invalid_FieldInput",$TypeDef);
		}
	}
	else {
			ThrowValidationException("CQCLI_Invalid_FieldInput",$TypeDef);
	}
}


sub ValidateRecordIds()
{
	 
	my ($keys1 , $keys2) = (undef,undef);
	my $id = undef ;
	
	$keys1 = $RIdDef;
	$keys2 = $DbidDef;


	if( !($keys1) && !($keys2) && IsAPIMode()) {
		ThrowValidationException('CQCLI_Failure_MissingIds');
	}
	elsif( !($keys1) && !($keys2))
	{
		if(!CheckPromptModeValidity('CQCLI_Missing_IdsWarn') && AskForYN('CQCLI_Prompt_AskRid'))
		{
				$id = GetRecordId();
				$RIdDef= $id;		
		}
		elsif(!CheckPromptModeValidity('CQCLI_Missing_IdsWarn') && AskForYN('CQCLI_Prompt_AskDbid'))
		{
				$id = GetDbId();
				$DbidDef = $id;
		}
		$keys1 = $RIdDef;
		$keys2 = $DbidDef;
		if( !($keys1) && !($keys2)) {
			ThrowValidationException('CQCLI_Failure_MissingIds');
		}
	}

	SetRecordIds();
}

#################################################################################################################



sub SetRecordIds
{
	my ($rectypeinfo,$key,$flag,$ridhash,$dbidhash) = (undef,undef,undef,undef,undef);
	try {
		$rectypeinfo = GetRecordInfoObj();
		$flag=1;

		$ridhash= GetJavaHash();
		$ridhash->put($RIdDef,$TypeDef);

		$rectypeinfo->checkValidity($ridhash,$flag);
		$JavaReqObj->put("rid",$RIdDef);
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Failure_RId");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}finally {};
}


sub Parse()
{
	my($filehash,$key) = (undef,undef);
	
	try {
		parseForHelpUsage();
		parseForInterActiveOption();
		parseForLoginCredentials();
		parseForCoreOptions();
		parseForUnknownOptions();
		$JavaReqObj->put("action",$ActionDef);
		$JavaReqObj->put("fldname",$FieldName);
		$JavaReqObj->put("rid",$RIdDef);
	
		if(keys %FileCommentAssociation) {
			$filehash= GetJavaHash();
			foreach $key (keys %FileCommentAssociation) {
				$filehash->put($key,$FileCommentAssociation{$key});
			}
		
			$JavaReqObj->put("attach",$filehash);
		}	

	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
		
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Validate()
{
	try {

		ValidateSessionInfo();
		ValidateRecordIds();
		ValidateActionNames();
		ValidateFieldNames();
		
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub addattachment
{
	my @tmpCmd = @_;
	my $req = undef;
	try {

		InitParser(@tmpCmd);
		Parse();
		Validate();

		$req =  GetJavaPayload("add_attachment",$JavaReqObj);

		SetRequestObject($req);
		return 1;
	}finally {
		CleanUpParser();
	};
}



1;
